@@ -13,8 +13,9 @@ from reviewboard.reviews.signals import (review_request_publishing,
 from reviewboard.site.urlresolvers import local_site_reverse
 
 from rbbz.bugzilla import Bugzilla
+from rbbz.diffs import build_plaintext_review
 from rbbz.errors import (BugzillaError, ConfidentialBugError, InvalidBugsError,
-                         InvalidBugIdError, InvalidReviewersError)
+                         InvalidBugIdError)
 
 def review_request_url(review_request, site=None, siteconfig=None):
     if not site:
@@ -56,36 +57,29 @@ def publish_review_request(user, review_request_draft, **kwargs):
     if b.is_bug_confidential(bug_id):
         raise ConfidentialBugError
 
-    reviewers = review_request_draft.target_people
-    num_reviewers = reviewers.count()
+    reviewers = [x.get_username() for x in
+                 review_request_draft.target_people.all()]
 
-    if num_reviewers == 0 or num_reviewers > 1:
-        raise InvalidReviewersError
-
-    reviewer = reviewers.first()
     b.post_rb_url(bug_id,
                   review_request_draft.get_review_request().id,
                   review_request_draft.summary,
                   review_request_draft.description,
                   review_request_url(review_request_draft.get_review_request()),
-                  reviewer.get_username())
+                  reviewers)
 
 
 def publish_review(user, review, **kwargs):
+    bug_id = int(review.review_request.get_bug_list()[0])
+    site = Site.objects.get_current()
+    siteconfig = SiteConfiguration.objects.get_current()
+
+    b = Bugzilla(user.bzlogin, user.bzcookie)
+    b.post_comment(bug_id, build_plaintext_review(review, {"user": user}))
+
     if review.ship_it:
-        bug_id = int(review.review_request.get_bug_list()[0])
-        site = Site.objects.get_current()
-        siteconfig = SiteConfiguration.objects.get_current()
-
-        b = Bugzilla(user.bzlogin, user.bzcookie)
-        attachments = b.get_rb_attachments(bug_id)
-
-        for a in attachments:
-            if (a['reviewer'] == review.user.username and
-                a['url'] == review_request_url(review.review_request, site,
-                                               siteconfig)):
-                b.r_plus_attachment(a['id'])
-                break
+        b.r_plus_attachment(bug_id, review.user.username,
+                            review_request_url(review.review_request, site,
+                                               siteconfig))
 
 
 def publish_reply(user, reply, **kwargs):
